-- $Id: DualSpeccer.lua 22 2012-09-01 13:33:59Z DrKazza $
local addon = LibStub("AceAddon-3.0"):NewAddon("DualSpeccer", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
DualSpeccer = addon

-- Blizzard constants for this have "Talents" next to it which looks ugly and redundant
local groupNames = {"Primary", "Secondary"}

local primarySpecSpellName = GetSpellInfo(TALENT_ACTIVATION_SPELLS[1])
local secondarySpecSpellName = GetSpellInfo(TALENT_ACTIVATION_SPELLS[2])

local activeGroup = nil
local specCache = {}

local filtering = false
local lastSwitchStartTime = 0

local rightClickMenu = {
	{
		text = "Hide switch spam",
		func = function() addon.db.profile.filterSpam = not addon.db.profile.filterSpam; addon:Update() end,
		checked = function() return addon.db.profile.filterSpam end,
		tooltipTitle = "Hide switch spam",
		tooltipText = "Hides the spell learned/unlearned messages when swapping.",
		isNotRadio = true
	},
	{
		text = "Show specialization name",
		func = function() addon.db.profile.showSpecName = not addon.db.profile.showSpecName; addon:Update() end,
		checked = function() return addon.db.profile.showSpecName end,
		tooltipTitle = "Show specialization name",
		tooltipText = "Displays the name of the specialization instead of primary/secondary.",
		isNotRadio = true
	},
	{
		text = "Give custom name",
		func = function() StaticPopup_Show("DualSpeccerCustomName") end,
		tooltipTitle = "Give custom name",
		tooltipText = "Allows you to give a custom name to your specialization.",
		notCheckable = true
	},
}

-- Match patterns we use. Replace any "." with "%." and any "%s" with "(.*)".
local spamFilterMatch1 = string.gsub(ERR_LEARN_ABILITY_S:gsub('%.', '%.'), '%%s', '(.*)')
local spamFilterMatch2 = string.gsub(ERR_LEARN_SPELL_S:gsub('%.', '%.'), '%%s', '(.*)')
local spamFilterMatch3 = string.gsub(ERR_SPELL_UNLEARNED_S:gsub('%.', '%.'), '%%s', '(.*)')
local spamFilterMatch4 = string.gsub(ERR_LEARN_PASSIVE_S:gsub('%.', '%.'), '%%s', '(.*)')

-- Upvalues
local GetNumTalentGroups, GetActiveTalentGroup, GetTime, GetPrimaryTalentTree, GetTalentTabInfo =
	GetNumTalentGroups, GetActiveTalentGroup, GetTime, GetPrimaryTalentTree, GetTalentTabInfo

local feed = LibStub("LibDataBroker-1.1"):NewDataObject("DualSpeccer", {
	type = "data source",
	text = "DualSpeccer",
	icon = "Interface\\Icons\\Ability_Marksmanship",
	tocname = "DualSpeccer"
})

function addon:OnInitialize()
	local dbDefaults = {
		profile = {
			filterSpam = true,
			showSpecName = false
		}
	}
	self.db = LibStub("AceDB-3.0"):New("DualSpeccerDB", dbDefaults, "Default")
	self.db.char.nameOverrides = self.db.char.nameOverrides or {}
	
	StaticPopupDialogs["DualSpeccerCustomName"] = {
		hasEditBox = 1,
		text = "Enter a custom name for your active specialization:",
		button1 = ACCEPT,
		button2 = CANCEL,
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1,
		maxLetters = 20,
		OnShow = function(self)
			self.editBox:SetText(addon.db.char.nameOverrides[activeGroup] or "")
			self.editBox:SetFocus()
		end,
		OnHide = function(self)
			self.editBox:SetText("")
		end,
		OnAccept = function(self)
			local text = self.editBox:GetText()
			if text == "" then text = nil end
			addon.db.char.nameOverrides[activeGroup] = text
			self:Hide()
			addon:Update()
		end,
		EditBoxOnEnterPressed = function(self)
			local text = self:GetParent().editBox:GetText()
			if text == "" then text = nil end
			addon.db.char.nameOverrides[activeGroup] = text
			self:GetParent():Hide()
			addon:Update()
		end,
		EditBoxOnEscapePressed = function(self)
			self:GetParent():Hide()
		end,
	}
end

function addon:OnEnable()
	self:RegisterEvent("UNIT_SPELLCAST_START")
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	self:RegisterEvent("PLAYER_LEAVING_WORLD")
	
	-- Chances are good that the P_E_W will fire after we're done, registering PLAYER_TALENT_UPDATE twice. This
	-- is more of an AceAddon implementation thing and CallbackHandler overwrites it anyway.
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_TALENT_UPDATE")
	
	-- The important part here is that it happens after the first OnUpdate which is seemingly the time when correct
	-- information becomes available.
	self:ScheduleTimer("Update", 0.01)
end

-- A "corrupted" talent update fires between LEAVING and ENTERING when zoning. We don't want this one.
function addon:PLAYER_LEAVING_WORLD(event)
	self:UnregisterEvent("PLAYER_TALENT_UPDATE")
end

function addon:PLAYER_ENTERING_WORLD(event)
	self:RegisterEvent("PLAYER_TALENT_UPDATE")
end

function addon:ACTIVE_TALENT_GROUP_CHANGED(event, newGroup)
	activeGroup = newGroup
	local name = specCache[newGroup].name or "unknown"
	self:Print(("Switched to %s."):format(name))
end

function addon:UNIT_SPELLCAST_START(event, ...)
	local unit, spellName = ...
	if unit == "player" and (spellName == primarySpecSpellName or spellName == secondarySpecSpellName) then
		lastSwitchStartTime = GetTime()
	end
end

function addon:PLAYER_TALENT_UPDATE(event)
	self:Update()
end

-- Our chat filter function.
local function filter(self, event, msg)
	-- You have unlearned Blahblah.
	-- You have learned a new spell: Blahblah.
	-- You have learned a new ability: Blahblah.
	if (lastSwitchStartTime > GetTime() - 8) and (msg:find(spamFilterMatch1) ~= nil or msg:find(spamFilterMatch2) ~= nil or msg:find(spamFilterMatch3) ~= nil or msg:find(spamFilterMatch4) ~= nil) then
		return true
	end
end

-- Updates our talent information.
function addon:Update()
	for i = 1, GetNumSpecGroups() do
		specCache[i] = specCache[i] or {}
		local thisCache = specCache[i]
		
		-- This fetches the currently selected specialization (which can have 0 points).
		-- We now use the selected specialization instead of the tree with the most points.
--THIS NEEDS FIXING BUT I CAN'T FIND OUT HOW TO INSPECT SECONDARY TALENT SPEC
--		local mainTree = GetSpecialization(i)
--		if mainTree ~= nil then
--			local _, name, _, icon = GetSpecializationInfo(mainTree)
--			local pointsSpent = GetNumTalents(i, false, false)
--			thisCache.name = name
--			thisCache.icon = icon	
			-- Fill points for all specs
--			for tree = 1, 3 do
--				local pointsSpent = GetNumTalents(tree, false, false, i)
--				thisCache[tree] = pointsSpent
--			end
--			thisCache[1] = pointsSpent
--			thisCache[2] = 0
--			thisCache[3] = 0
--		else
--			thisCache.name = "None"
--			-- Hardcoded in a local variable in TalentUI (specs.spec1.defaultSpecTexture).
--			thisCache.icon = "Interface\\Icons\\Ability_Marksmanship"
--			thisCache[1] = 0
--			thisCache[2] = 0
--			thisCache[3] = 0
--		end
		if i == 1 then
			thisCache.name = "Primary"
		else
			thisCache.name = "Secondary"
		end	
	
		-- Apply it here so we don't have to do it everywhere.
		if self.db.char.nameOverrides[i] then
			thisCache.name = self.db.char.nameOverrides[i]
		end
		
		thisCache.groupName = groupNames[i]
	end
	
	activeGroup = GetActiveSpecGroup()
	local curCache = specCache[activeGroup]
	
	if self.db.profile.showSpecName then
		feed.text = curCache.name
	else
		feed.text = curCache.groupName
	end
	
	feed.icon = curCache.icon
	
	if not filtering and self.db.profile.filterSpam then
		ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", filter)
	elseif filtering and not self.db.profile.filterSpam then
		ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", filter)
	end
end

-- Menu anchoring stuff stolen from ClosetGnome.
local menuPopupFrame = CreateFrame("Frame", "DualSpeccerMenu", UIParent, "UIDropDownMenuTemplate")
function feed.OnClick(displayFrame, button)
	button = button or "LeftButton"
	
	if button == "LeftButton" then
		if GetNumSpecGroups() < 2 then
			addon:Print("This character does not have a second specialization.")
			return
		end
		
		-- -1 + 3 = 2 and -2 + 3 = 1.
		local newSpec = -activeGroup + 3
		SetActiveSpecGroup(newSpec)
	elseif button == "RightButton" then
		EasyMenu(rightClickMenu, menuPopupFrame, displayFrame, 20, 4, "MENU")
	end
end

function feed.OnTooltipShow(tooltip)
	tooltip:AddLine("DualSpeccer")
	local curCache = specCache[curGroup]
	
	if not curCache then
		addon:Update()
		curCache = specCache[activeGroup]
		-- There used to be a check here, but if curCache is still nil after that, we have Bigger Problems.
	end
	
	tooltip:AddDoubleLine("Active", ("%s (%s)"):format(curCache.groupName, curCache.name))
	
	for i = 1, GetNumSpecGroups() do
		local specColor = (activeGroup == i) and "6adb54" or "ffff99"
--		tooltip:AddDoubleLine(groupNames[i], ("|cff%s%s|r (%d/%d/%d)"):format(specColor, specCache[i].name, specCache[i][1], specCache[i][2], specCache[i][3]))
		tooltip:AddDoubleLine(groupNames[i], ("|cff%s%s|r"):format(specColor, specCache[i].name))
	end
	
	if GetNumSpecGroups() >= 2 then
		tooltip:AddLine(" ")
		tooltip:AddLine("|cffeda55fClick|r to switch to your other specialization.")
	end
	
	tooltip:AddLine("|cffeda55fRight-Click|r to open the menu.")
end

